/*
 
 Ce programme est optimisé pour Processing dans sa version 1.5.1
 
 Lycée Jean-Perrin
 
*/

//importation des librairies
import saito.objloader.*; // OBJ Loader
import java.io.*;
import processing.opengl.*;
import jp.nyatla.nyar4psg.*;
import codeanticode.gsvideo.*;

import ddf.minim.*; // librairies audio minim
import ddf.minim.signals.*; // librairies audio minim
import ddf.minim.analysis.*; // librairies audio minim
import ddf.minim.effects.*; // librairies audio minim

//initialisation des variables
GSCapture cam;
MultiMarker nya_1; // variable 1
MultiMarker nya_2; // variable 2
MultiMarker nya_3; // variable 3
int fps = 24;  // définit le nombre de fps (frame per second)
GSMovieMaker  mm;  //variable pour l'enregistrement du flux vidéo

OBJModel model; // objet pour OBJ Loader

Minim minim; // minim
AudioInput in; // minim 

int resX = 640; // résolution webcam x
int resY = 480; // résolution webcam y 

void setup() {
  
  // enregistrement vidéo 
  mm = new GSMovieMaker (this, width, height, "drawing2.ogg", GSMovieMaker.THEORA, GSMovieMaker.MEDIUM, fps);
  mm.start(); // créé le fichier "drawing1.ogg" et commence l'enregistrement  
  
  size(resX, resY, OPENGL); // définition de la taille de la fenêtre d'affichage
  // résolutions possibles avec webcam : 1280, 960 || 640, 480 || 800, 600 || 1024, 576 || 1280, 720 ... et bien d'autres  
  smooth();
  colorMode(RGB, 100); // définit le mode de couleur interprêté par processing, en RGB ici
  println(MultiMarker.VERSION); // affiche dans la zone de texte l'évolution d'un type de variable
  
  cam=new GSCapture(this, resX, resY); // définit la taille de capture du flux
  cam.start(); // lance l'aquisition du flux vidéo
  
  // Mettre de nom du fichier obj, si l'objet comprte trop de face, il ne s'affiche pas 
  model = new OBJModel(this, "objtestra3.obj", "absolute", QUADS);
  
  // -------------------------------------------------
  // ----- éléments pour bibliothèque obj loader -----
  // -------------------------------------------------
  
  model.enableDebug();
  model.enableTexture(); // texture activé 

  model.scale(1); // zoom sur l'objet : réglage en fonction taille souhaitée (si 0, pas d'objet visible)
  model.translateToCenter();
    
  // --------------------------------------------------
  // ----- éléments pour bibliothèque audio minim -----
  // --------------------------------------------------
  
  minim = new Minim(this);
  in = minim.getLineIn(); // récupérer le son du micro
  
  // -----------------------------------------------------------------------
  // --- déclaration des markers et association d'un marker à un pattern ---
  // -----------------------------------------------------------------------
  
  // marker1
  nya_1=new MultiMarker(this, width, height, "camera_para.dat", new NyAR4PsgConfig(NyAR4PsgConfig.CS_LEFT_HAND, NyAR4PsgConfig.TM_NYARTK));
  nya_1.addARMarker("m1.patt", 80);

  // marker2
  nya_2=new MultiMarker(this, width, height, "camera_para.dat", new NyAR4PsgConfig(NyAR4PsgConfig.CS_RIGHT_HAND, NyAR4PsgConfig.TM_NYARTK));
  nya_2.addARMarker("m2.patt", 80);
  
  frameRate(20); // mise a 20 du frameRate pour la capture vidéo
}

float r; // variable pour rotation objets 3D

void drawgrid(){ //implemente un quadrillage

  pushMatrix(); //sauvegarde les changements de coordonnées
    stroke(0); //définit la couleur utilisée pour dessiner des lignes et des bordures autour des formes (en RGB de 0 à 255)
    strokeWeight(2); //définit la taille des lignes et des bordures autour des formes
  popMatrix(); //réinitialise les changements de coordonées
}

void draw(){ //implemente le dessin des objets liés aux markers

  if (cam.available() !=true) { // permet de vérifier la présence d'une webcam
    return;
  }

  // -------------------------------------------------
  // ----- initialisation des markers à détecter -----
  // -------------------------------------------------
  
  cam.read(); // lance la lecture du flux
  image(cam, 0, 0, resX, resY);
  
  background(0); //définit la couleur de la fenêtre processing en RGB (valeur de 0 à 255)
  
  nya_1.detect(cam);  // détection du marker1
  nya_2.detect(cam);  // détection du marker2

  // ------------------------------------------------
  // ----- initialisation d'une police de texte -----
  // ------------------------------------------------

  PFont w; //définition de la variable 
  w = createFont("Comic sans MS", 35, true); //affectation des valeurs de la variable lié au modification de texte

  nya_1.drawBackground(cam);
  nya_2.drawBackground(cam);
  
  // ----------------------------------
  // ----- affichage sur marker 1 -----
  // ----------------------------------
  
  if ((nya_1.isExistMarker(0))) { //fonction "si", si le marqueur 1 est présent executé la suite.
    nya_1.beginTransform(0); //début des modifications virtuelles
    fill(0, 0, 255); //permet le changement de couleur de fonction (r,g,b)
    drawgrid();
    
    lights();
    
    rotateX(-PI/2);
    pushMatrix();
      fill(0,0,255);
      noStroke();
      translate(0, -30, 0);//gère l'emplacement de l'objet par rapport au marker
      r += 0.05;
      r = r % (PI*2.0);
      // faire une rotation avec valeur r
      rotateY(r);
      //rotateZ(-r);
      model.draw();
    popMatrix();
    
    nya_1.endTransform(); //fin des modifications virtuelles
  }
  
  // ----------------------------------
  // ----- affichage sur marker 2 -----
  // ----------------------------------

  if ((nya_2.isExistMarker(0))) {
    nya_2.beginTransform(0);
    translate(0,0,40);
    pushMatrix();
    rotateX(-PI/2);
    rotateY(PI/4);
    translate(-60,0,0);
    for(int i = 0; i < 120 - 1; i++){
      float var2 = 0;
      int var3 = 0;
      stroke(0,0,255,100);//le 4ième paramètre rajoute de la transparence
      strokeWeight(1.3);
      line( i, (0) + in.left.get(i)*380, i+1, (0) + in.left.get(i+1)*380 );//en diminuant les 380, on diminue la sensibilité au son
      line(60,40,i,in.left.get(i)*380);
      strokeWeight(1);
      float var1 = 384 + in.left.get(i)*450;
    }
    popMatrix();
    nya_2.endTransform();
  }

  loadPixels(); // charge les pixels pour enregistrer le flux vidéo
  mm.addFrame(pixels); // ajoute les images au fichier vidéo depuis les pixels chargés
} 

// --------------------------------------
// --- arret script avec barre espace ---
// --------------------------------------

void keyPressed(){ // fonction qui permet d'utiliser le clavier
  if (key == ' ') // associe la fonction suivante à la barre d'espace
  {
    mm.finish(); //stoppe l'enregistrement si la barre d'espace est appuyée
    exit(); // quitte le programme une fois le fichier écrit
  }
} //fin du programme

